

class SettingsEvent(object):
    """ """
    def __init__(self, module=None):
        self.module = module


class SettingsBeforeSave(SettingsEvent):
    pass


class SettingsAfterSave(SettingsEvent):
    pass



import colander
import deform

from kotti_controlpanel import _

# hold the setting objects
SETTINGS = {}
CONTROL_PANEL_LINKS = {}

# Here we define some default schemas.
slot_names = (('left', _(u'left')),
              ('right', _(u'right')),
              ('abovecontent', _(u'abovecontent')),
              ('belowcontent', _(u'belowcontent')),
              ('beforebodyend', _(u'beforebodyend')),)


class SlotSchemaNode(colander.SchemaNode):
    name = 'slot'
    title = _(u'Direction')
    default = u'left'
    widget = deform.widget.SelectWidget(values=slot_names)


show_in_context = (('everywhere', _(u'Everywhere')),
                   ('only on root', _(u'Only on root')),
                   ('not on root', _(u'Not on root')),
                   ('nowhere', _(u'Nowhere')))


class ShowInContextSchemaNode(colander.SchemaNode):
    name = 'show_in_context'
    title = _(u'Show in context')
    default = u'everywhere'
    widget = deform.widget.SelectWidget(values=show_in_context)



# # -*- coding: utf-8 -*-
#
# """
# Created on 2016-06-15
# :author: Oshane Bailey (b4.oshany@gmail.com)
# """
#
# import colander
# import deform
# from kotti.views.edit import ContentSchema
# from kotti.views.form import AddFormView
# from kotti.views.form import EditFormView
# from pyramid.view import view_config
#
# from kotti_controlpanel import _
# from kotti_controlpanel.resources import ControlPanelPage
#
#
# class ControlPanelPageSchema(ContentSchema):
#     """ Schema for CustomContent. """
#     title = colander.SchemaNode(
#         colander.String(),
#         title=_(u'Settings for:'),
#     )
#
#
# @view_config(name=ControlPanelPage.type_info.add_view,
#              permission=ControlPanelPage.type_info.add_permission,
#              renderer='kotti:templates/edit/node.pt')
# class ControlPanelPageAddForm(AddFormView):
#     """ Form to add a new instance of CustomContent. """
#
#     schema_factory = ControlPanelPageSchema
#     add = ControlPanelPage
#     item_type = _(u"System Settings")
#
#     def save_success(self, appstruct):
#         title = appstruct["title"]
#         if not title.lower().endswith("settings"):
#             appstruct["title"] = "{} Settings".format(title)
#         appstruct["editor_name"] = "{}".format(self.request.user.name)
#         return super(ControlPanelPageAddForm, self).save_success(appstruct)
#
#
# @view_config(name='edit', context=ControlPanelPage, permission='edit',
#              renderer='kotti:templates/edit/node.pt')
# class ControlPanelPageEditForm(EditFormView):
#     """ Form to edit existing CustomContent objects. """
#
#     schema_factory = ControlPanelPageSchema
#
#     def save_success(self, appstruct):
#         appstruct["editor_name"] = "{}".format(self.request.user.name)
#         return super(ControlPanelPageEditForm, self).save_success(appstruct)



# -*- coding: utf-8 -*-
import sys
import colander
import deform
from types import ClassType
from pyramid.decorator import reify
from pyramid_deform import CSRFSchema
from pyramid_deform import FormView
from pyramid.httpexceptions import HTTPFound

from kotti_controlpanel.events import SettingsAfterSave
from kotti_controlpanel.events import SettingsBeforeSave
from kotti_controlpanel.util import get_settings
from kotti_controlpanel import _

import itertools
counter = itertools.count()


class SettingsSchema(colander.MappingSchema):
    """An empty schema to have a named one if needed.
    """
    pass


class SettingsFormView(FormView):
    """The form template class for one setting tab.
    """
    form_class = deform.form.Form
    name = 'settings'
    title = _(u"Settings")
    description = u""
    success_message = _(u"Your changes have been saved.")
    success_url = None
    settings = None
    schema_factory = None
    use_csrf_token = True
    use_ajax = True
    active = False
    form_options = {'counter': counter}

    def __init__(self, context, request, **kwargs):
        self.context = context
        self.request = request
        self.__dict__.update(kwargs)

    def __call__(self):
        """Build up the schema and return the form view.
        """
        # build the schema if it not exist
        if self.schema is None:
            if self.schema_factory is None:
                self.schema_factory = SettingsSchema
            self.schema = self.schema_factory()
        # append csrf token if needed
        if self.use_csrf_token and 'csrf_token' not in self.schema:
            self.schema.children.append(CSRFSchema()['csrf_token'])
        # enhance the schema with the given definitions
        for setting_obj in self.settings.settings_objs:
            node = colander.SchemaNode(
                self.colander_type(setting_obj.type)(),
                name=setting_obj.field_name,
                title=setting_obj.title,
                description=setting_obj.description,
                default=setting_obj.default
            )
            self.schema.children.append(node)
        # the names of the children should begin with the module name
        for child in self.schema.children:
            if child.name == 'csrf_token':
                continue
            if not child.name.startswith(self.settings.module):
                child.name = "%s-%s" % (self.settings.module, child.name)
        # Build up the buttons dynamically, so we can check what setting form
        # was saved.
        save = 'save_' + self.form_id
        self.buttons = (
            deform.Button(save, _(u'Save')),
            deform.Button('cancel', _(u'Cancel')))
        setattr(self, save + '_success', self.save_success)
        return super(SettingsFormView, self).__call__()

    @reify
    def form_id(self):
        form_id = "{0}-{1}".format(self.settings.module, self.name)
        return form_id

    def before(self, form):
        settings = get_settings()
        for key in form.cstruct:
            if key in settings:
                # Convert boolean to 'true' or 'false' to meet the
                # requirements of deform's checkbox widget.
                if isinstance(settings[key], bool):
                    value = settings[key] and 'true' or 'false'
                else:
                    value = settings[key]
                form.cstruct[key] = value
        form.formid = self.form_id

    def colander_type(self, name):
        # http://docs.pylonsproject.org/projects/colander/en/latest/api.html#types
        try:
            klass = getattr(sys.modules['colander'], name)
        except AttributeError:
            raise NameError("%s is not an sqlalchemy data type." % name)
        if isinstance(klass, (ClassType, type)):
            return klass
        raise TypeError("%s is not a class." % name)  # pragma: no cover

    def save_success(self, appstruct):
        from kotti_controlpanel.util import set_setting
        formid = self.request.POST.get('__formid__', None)
        self.active = False
        if formid is not None and formid == self.form_id:
            self.active = True
            appstruct.pop('csrf_token', None)
            module = None
            if appstruct:
                key = appstruct.keys()[0]
                module = key[:key.find('-')]
            self.request.registry.notify(SettingsBeforeSave(module))
            for item in appstruct:
                if appstruct[item] is not None:
                    set_setting(item, appstruct[item])
            ses = self.request.session
            if not '_f_success' in ses or\
                not self.success_message in ses['_f_success']:
                    self.request.session.flash(self.success_message, 'success')
            self.request.registry.notify(SettingsAfterSave(module))

    def cancel_success(self, appstruct):
        self.request.session.flash(_(u'No changes made.'), 'info')
        location = "%s@@settings" % self.request.resource_url(self.context)
        raise HTTPFound(location=location)



# -*- coding: utf-8 -*-

"""
Created on 2016-06-15
:author: Oshane Bailey (b4.oshany@gmail.com)
"""

from pyramid.i18n import TranslationStringFactory
from kotti.util import Link
from kotti.views.site_setup import CONTROL_PANEL_LINKS

_ = TranslationStringFactory('kotti_controlpanel')


def kotti_configure(settings):
    """ Add a line like this to you .ini file::

            kotti.configurators =
                kotti_controlpanel.kotti_configure

        to enable the ``kotti_controlpanel`` add-on.

    :param settings: Kotti configuration dictionary.
    :type settings: dict
    """

    settings['pyramid.includes'] += ' kotti_controlpanel'
    settings['kotti.alembic_dirs'] += ' kotti_controlpanel:alembic'
    # settings['kotti.available_types'] += (
    #     ' kotti_controlpanel.resources.ControlPanelPage'
    # )
    settings['kotti.fanstatic.view_needed'] += (
        ' kotti_controlpanel.fanstatic.css_and_js'
    )

    settings = Link('controlpanel', title=_(u'Control Panel'))
    CONTROL_PANEL_LINKS.append(settings)

def includeme(config):
    """ Don't add this to your ``pyramid_includes``, but add the
    ``kotti_configure`` above to your ``kotti.configurators`` instead.

    :param config: Pyramid configurator object.
    :type config: :class:`pyramid.config.Configurator`
    """

    config.add_translation_dirs('kotti_controlpanel:locale')
    config.add_static_view('static-kotti_controlpanel', 'kotti_controlpanel:static')

    config.scan(__name__)



# -*- coding: utf-8 -*-

"""
Created on 2016-06-15
:author: Oshane Bailey (b4.oshany@gmail.com)
"""


class BaseView(object):
    """ Base class for views """

    def __init__(self, context, request):
        """ Constructor

        :param context: Context of the view
        :type context: :class:`kotti.resources.Content`

        :param request: Current request
        :type request: :class:`kotti.request.Request`
        """

        super(BaseView, self).__init__()

        self.context = context
        self.request = request



# -*- coding: utf-8 -*-

"""
Created on 2016-06-15
:author: Oshane Bailey (b4.oshany@gmail.com)
"""
import json

from kotti import Base
from kotti.interfaces import IDefaultWorkflow
from kotti.resources import Content
from sqlalchemy import (
    Column,
    String,
    Unicode,
    Integer,
    ForeignKey
)
from sqlalchemy.types import PickleType
from sqlalchemy.orm import relationship
from sqlalchemy.ext.declarative import declared_attr
from zope.interface import implements

from kotti_controlpanel import _


class ControlPanel(Base):

    tablename = 'controlpanels'

    @declared_attr
    def __tablename__(cls):
        return cls.tablename

    id = Column(Unicode(100), primary_key=True, nullable=False)
    settings = Column(PickleType)

    @classmethod
    def get_by_id(cls, id):
        return cls.query.filter(cls.id == id).first()

    def set_settings(self, settings):
        self.settings = settings

    def get_settings(self):
        return self.settings


class ModuleSettings(object):
    """Object to hold informations about the settings of
       the calling module.
    """

    def __init__(self, **kwargs):
        self.module = None
        self.name = None
        self.title = None
        self.description = None
        self.success_message = u''
        self.settings = []
        self.schema_factory = None
        self.settings_objs = []
        self.__dict__.update(kwargs)


class SettingObj(object):
    """One setting in a module. Here we are also set the defaults
       for one setting.
    """

    def __init__(self, **kwargs):
        self.module = None
        self.type = 'String'
        self.name = None
        self.title = None
        self.description = u''
        self.default = ''
        self.__dict__.update(kwargs)

    @property
    def field_name(self):
        """Return a name that is somewhat unique.
        """
        return "%s-%s" % (self.module, self.name)



# -*- coding: utf-8 -*-

"""
Created on 2016-06-15
:author: Oshane Bailey (b4.oshany@gmail.com)
"""

from pytest import fixture


@fixture
def dummy_content(root):

    from kotti_controlpanel.resources import CustomContent

    root['cc'] = cc = CustomContent(
        title=u'My content',
        description=u'My very custom content is custom',
        custom_attribute='Lorem ipsum'
    )

    return cc


def test_view(dummy_content, dummy_request):

    from kotti_controlpanel.views.view import CustomContentViews

    views = CustomContentViews(dummy_content, dummy_request)

    default = views.default_view()
    assert 'foo' in default

    alternative = views.alternative_view()
    assert alternative['foo'] == u'bar'



import re
import colander
import inspect

from kotti.resources import get_root
from kotti.views.slots import objectevent_listeners
from kotti.views.slots import slot_events

from kotti_controlpanel.config import SETTINGS, CONTROL_PANEL_LINKS
from kotti_controlpanel.resources import ControlPanel
from kotti_controlpanel.resources import ModuleSettings
from kotti_controlpanel.resources import SettingObj


def slugify(text, delim=u'-'):
    """Generates an slightly worse ASCII-only slug."""
    return re.sub("[^a-zA-Z\d:]", text.lower(), "-")


def get_setting(name, default=None, modname=None):
    if modname is None:
        frame = inspect.stack()[1]
        module = inspect.getmodule(frame[0])
        modname = module.__name__
        if '.' in modname:
            modname = modname[:modname.find('.')]  # pragma: no cover
    if not name.startswith(modname):
        name = '{0}-{1}'.format(modname, name)
    settings = get_settings()
    if name in settings:
        return settings[name]
    return default


def set_setting(name, val):
    settings = get_settings()
    # It seems not possible to save a set type in a MutableDict,
    # so convert it to a list.
    if type(val) == set:
        val = list(val)
    settings[name] = val


def get_settings():
    root = get_root()
    if "kotti_controlpanel" not in root.annotations:
        root.annotations['kotti_controlpanel'] = {}
    return root.annotations['kotti_controlpanel']


def get_links(controlpanel_id):
    cpi = slugify(controlpanel_id)
    return CONTROL_PANEL_LINKS.get(cpi, [])


def add_links(controlpanel_id, *links):
    cpi = slugify(controlpanel_id)
    clinks = CONTROL_PANEL_LINKS.get(cpi, [])
    for link in links:
        clinks.append(link)
    CONTROL_PANEL_LINKS[cpi] = clinks


def add_settings(mod_settings, links=None):
    """Get a dictionary and translate this into an object structure.
    """
    settings = get_settings()
    module_settings = ModuleSettings(**mod_settings)
    if module_settings.module is None:
        # If the module name is omitted: get it
        frame = inspect.stack()[1]
        module = inspect.getmodule(frame[0])
        modname = module.__name__
        if '.' in modname:
            modname = modname[:modname.find('.')]
        module_settings.module = modname
    if module_settings.settings:
        for setting in module_settings.settings:
            setting_obj = SettingObj(**setting)
            # name and title of a setting is required
            if setting_obj.name is None:
                raise ValueError('A setting has to have a name.')
            if setting_obj.title is None:
                raise ValueError('A setting has to have a title.')
            setting_obj.module = module_settings.module
            default = None
            if 'default' in setting:
                default = setting['default']
            if not setting_obj.field_name in settings:
                settings[setting_obj.field_name] = default
            module_settings.settings_objs.append(setting_obj)
    if module_settings.schema_factory:
        schema = module_settings.schema_factory()
        for child in schema.children:
            field_name = child.name
            if not field_name.startswith(module_settings.module):
                field_name = u"%s-%s" % (module_settings.module, child.name)
            if not field_name in settings:
                value = None
                if child.default is not colander.null:
                    value = child.default
                settings[field_name] = value
    mod_slug = slugify(module_settings.name)
    if type(links) == list:
        add_links(mod_slug, *links)
    module_settings.slug_id = mod_slug
    SETTINGS[mod_slug] = module_settings


def remove_from_slots(widget, slot=None):
    """Check all slots if a widget is already set and remove it
       from the listener.
    """
    for slot_event in slot_events:
        if slot is not None:
            if slot != slot_event.name:
                continue
        try:
            listener = objectevent_listeners[(slot_event, None)]
        except TypeError:  # pragma: no cover
            listener = None
        if listener is not None:
            for func in listener:
                for closure in func.func_closure:
                    if closure.cell_contents == widget:
                        listener.remove(func)
                        break


def show_in_context(setting, context):
    """Check if the the item should be shown in the given context."""
    show = False
    if setting == u'everywhere':
        show = True
    elif setting == u'only on root':
        show = context == get_root()
    elif setting == u'not on root':
        show = context != get_root()
    return show



# -*- coding: utf-8 -*-

"""
Created on 2016-06-15
:author: Oshane Bailey (b4.oshany@gmail.com)
"""

from pyramid.view import view_config
from pyramid.view import view_defaults
from pyramid import httpexceptions as httpexc
from pyramid.renderers import render_to_response

from kotti.views.util import is_root

from kotti_controlpanel.forms import SettingsFormView

from kotti import DBSession
from kotti.views import users as kotti_users
from kotti_controlpanel import _, util
from kotti_controlpanel.config import SETTINGS
from kotti_controlpanel.resources import ControlPanel
from kotti_controlpanel.fanstatic import css_and_js
from kotti_controlpanel.views import BaseView


class BaseSettingViews(BaseView):

    @view_config(name='controlpanel',
                 custom_predicates=(is_root, ),
                 permission='manage',
                #  request_method = 'GET',
                 renderer='kotti_controlpanel:templates/controlpanel.pt')
    def view(self):
        setting_id = self.request.params.get("setting_id")
        if not setting_id:
            settings_form_views = []
            for settings in SETTINGS.values():
                settings_form_views.append(
                    settings
                )
            return {
                "settings": settings_form_views
            }
        settings = SETTINGS.get(setting_id)
        if not settings:
            return httpexc.HTTPNotFound()
        args = {
            'title': settings.title,
            'description': settings.description,
            'name': settings.name,
            'schema_factory': settings.schema_factory,
            'settings': settings,
            'success_message': settings.success_message,
            'active': True,
        }
        View = type(str(setting_id), (SettingsFormView,), args)
        view = View(self.context, self.request)
        form = view()
        # import pdb; pdb.set_trace()
        form["view"] = view
        links = util.get_links(setting_id)
        return render_to_response(
            'kotti_controlpanel:templates/settings.pt',
            {
                "settings": settings,
                "settings_form": form,
                "cp_links": links
            },
            request=self.request)

    # @view_config(name='controlpanel',
    #              custom_predicates=(is_root, ),
    #              permission='manage',
    #              request_method = 'POST',
    #              renderer='kotti_controlpanel:templates/controlpanel.pt')
    # def post(self):
    #     setting_id = self.request.params.get("setting_id")



import os

from setuptools import setup
from setuptools import find_packages

here = os.path.abspath(os.path.dirname(__file__))
try:
    README = open(os.path.join(here, 'README.rst')).read()
except IOError:
    README = ''
try:
    CHANGES = open(os.path.join(here, 'CHANGES.rst')).read()
except IOError:
    CHANGES = ''

version = '0.0.2'

install_requires = [
    'Kotti>=1.0.0',
    'unidecode',
]


setup(
    name='kotti_controlpanel',
    version=version,
    description="Settings configuration for Kotti",
    long_description='\n\n'.join([README, CHANGES]),
    classifiers=[
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.7",
        "Framework :: Pylons",
        "Topic :: Internet :: WWW/HTTP",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
        "Topic :: Internet :: WWW/HTTP :: WSGI :: Application",
        "License :: Repoze Public License",
    ],
    author='Oshane Bailey',
    author_email='b4.oshany@gmail.com',
    url='https://github.com/b4oshany/kotti_controlpanel',
    keywords='ui settings controlpanel kotti web cms wcms pylons pyramid sqlalchemy bootstrap',
    license="BSD-derived (http://www.repoze.org/LICENSE.txt)",
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,
    install_requires=install_requires,
    tests_require=[],
    dependency_links=[],
    entry_points={
        'fanstatic.libraries': [
            'kotti_controlpanel = kotti_controlpanel.fanstatic:library',
        ],
    },
    package_data={"kotti_controlpanel": ["templates/*", "static/*",
                                          "locale/*", "views/*",
                                          "alembic/*.*",
                                          "alembic/versions/*"]},
    extras_require={},
)



# -*- coding: utf-8 -*-

"""
Created on 2016-06-15
:author: Oshane Bailey (b4.oshany@gmail.com)
"""

from __future__ import absolute_import

from fanstatic import Group
from fanstatic import Library
from fanstatic import Resource


library = Library("kotti_controlpanel", "static")

css = Resource(
    library,
    "styles.css",
    minified="styles.min.css")
js = Resource(
    library,
    "scripts.js",
    minified="scripts.min.js")

css_and_js = Group([css, js])



# -*- coding: utf-8 -*-

"""
Created on 2016-06-15
:author: Oshane Bailey (b4.oshany@gmail.com)
"""

pytest_plugins = "kotti"

from pytest import fixture


@fixture(scope='session')
def custom_settings():
    import kotti_controlpanel.resources
    kotti_controlpanel.resources  # make pyflakes happy
    return {
        'kotti.configurators': 'kotti_tinymce.kotti_configure '
                               'kotti_controlpanel.kotti_configure'}
